class basic_agent #(type VIF = int, type T = basic_tr) extends uvm_agent;
  typedef basic_agent #(VIF, T) this_t;
  typedef basic_sequencer #(T) sequencer_t;
  typedef basic_driver #(VIF, T) driver_t;
  typedef basic_monitor #(VIF, T) monitor_t;

  sequencer_t sqr;
  driver_t    drv;
  monitor_t   mon;

  uvm_analysis_port #(T) item_collected_port;
  
  // Some example agents (apb_agent and partially vip_agent) will instantiate a
  // virtual interface here and propagate it down to lower components. Most of
  // the time there is simply no need to do that. It is just extra code for a
  // very small gain. We will avoid it here.
  
  `uvm_component_param_utils(this_t)

  function new(string name, uvm_component parent = null);
    super.new(name, parent);
    item_collected_port = new("item_collected_port", this);
  endfunction

  virtual function void build_phase(uvm_phase phase);
    super.build_phase(phase);
    mon = monitor_t::type_id::create("mon", this);

    // Only instantiate if active as the components otherwise may interfere with test execution
    if (get_is_active() == UVM_ACTIVE) begin
      sqr = sequencer_t::type_id::create("sqr", this);
      drv = driver_t::type_id::create("drv", this);
    end    
  endfunction

  virtual function void connect_phase(uvm_phase phase);
    super.connect_phase(phase);
    mon.item_collected_port.connect(item_collected_port);
    if (get_is_active() == UVM_ACTIVE) begin
      drv.seq_item_port.connect(sqr.seq_item_export);
    end
  endfunction
  
endclass
