class basic_driver #(type VIF = int, type T = basic_tr) extends uvm_driver #(T);
  typedef basic_driver #(VIF, T) this_t;
  // protected VIF vif;
  VIF vif;

  `uvm_component_param_utils(this_t)

  function new(string name, uvm_component parent = null);
    super.new(name, parent);
  endfunction

  virtual function void build_phase(uvm_phase phase);
    super.build_phase(phase);
    if(!uvm_config_db#(VIF)::get(this, "", "vif", vif))
      `uvm_error("NOVIF",{"virtual interface must be set for: ",get_full_name(),".vif"})
  endfunction

  // Why a run_phase and not the run-time phasing? The UVM creators recommend
  // that you make drivers and monitors independent and thus unaware of the
  // run-time phasing, so that they will work in all run-time phases for
  // example in the configuration phase or when users add their own phases.
  //
  // This recommendation is nowhere to be found UVM 1.1, but the recommendation
  // is reasonable and we will follow convention.
  virtual task run_phase(uvm_phase phase);
    super.run_phase(phase);
    vif.en <= 'x;
    vif.data <= 'x;

    fork
      reset_signals();
      get_and_drive(phase);
    join
  endtask

  virtual protected task reset_signals();
    wait (vif.resetb === 0);
    forever begin
      vif.en <= '0;
      vif.data <= 'x;
      @(negedge vif.resetb);
    end
  endtask

  virtual protected task get_and_drive(uvm_phase phase);
    wait(vif.resetb === 0);
    @(posedge vif.resetb);
    forever begin
      @(posedge vif.clk);
      vif.en <= '0;
      seq_item_port.get_next_item(req);
      $cast(rsp, req.clone());
      rsp.set_id_info(req);
      drive_transfer(rsp);
      seq_item_port.item_done();
      seq_item_port.put_response(rsp);
    end
  endtask

  virtual protected task drive_transfer(T tr);
    repeat (tr.latency) @(posedge vif.clk);
    vif.en <= '1;
    vif.data <= tr.sample;
  endtask
endclass
