`include "uvm_macros.svh"

// This package defines:
//   1. A parameterized transaction basic_tr
//   2. A parameterized agent basic_agent
//
// The transaction is parameterized according to the type of data transmitted.
// It is recommended that the user derive his custom transaction from a
// specialization of basic_tr, instead of typedefing it directly. This will be
// useful if one is willing to use a factory type override on the custom
// transactions.
//
// The agent is parameterized according to the virtual interface type and the
// transaction type. The interface should include signals "en" and "data", the
// transaction should have the fields in basic_tr (which is ensured if its
// derived from it), and the field 'sample' of the latter should be assignment
// compatible with the signal 'data' in the interface.
//
// The agent's specialization may be derived from or used directly, e.g.,
//   typedef basic_agent #(my_vif, my_tr) my_agent;
//
// The subcomponent types are conveniently accessible via typedefs. The agent
// has the following typedefs: 'sequencer_t', 'driver_t' and 'monitor_t', which
// may be used, e.g., as
//   typedef my_agent::sequencer_t my_sequencer;
package basic_pkg;

  import uvm_pkg::*;

  `include "basic_tr.sv"

  // There is no difference between a master, slave or bus monitor, so I will take a shortcut here
  `include "basic_monitor.sv"
  // There is no slave driver so there is no need to distinguish between master and slave
  `include "basic_sequencer.sv"
  `include "basic_driver.sv"

  // There is no difference between a passive master agent and a slave agent, so there is only one agent
  `include "basic_agent.sv"
endpackage
