// A linear PCM stream is completely specified by the samples, the resolution
// and the sampling frequency. Typically this information is known above the
// level of individual samples, so basic_tr will only encode the sample
// information and the latency from the previous sample in system clock cycles.
class basic_tr #(type T = int) extends uvm_sequence_item;
  typedef basic_tr #(T) this_t;

  // Most lpcm signals have a 16 bit (audio CD) or 24 bit (professional audio)
  // resolution, but other resolutions up to about 30 bit has been observed on
  // audiophile friends' stereos. The format is typically a fixed point
  // 1.15 or 1.23, but we don't care here as we're not going to perform any
  // arithmetic on the samples. 32 bit should be enough for everyone.
  rand T sample;
  rand int unsigned latency; // zero means back-to-back

  constraint valid_c {
    latency < 1_000_000;
  }

  // UVM_DEFAULT seems to be preferred over UVM_ALL_ON. UVM_ALL_ON is not
  // documented in UVM-1.1 so I take it that UVM_ALL_ON should not be used.
  //
  // If I read uvm_object_globals.svh right, then in UVM-1.1 UVM_DEFAULT will
  // do deep copying while UVM_ALL_ON will not.
  `uvm_object_param_utils_begin(this_t) 
    `uvm_field_int(sample,  UVM_DEFAULT+UVM_DEC)
    `uvm_field_int(latency, UVM_DEFAULT+UVM_DEC)
  `uvm_object_utils_end

  function new (string name = "basic_tr");
    super.new(name);
  endfunction

  function string convert2string();
    return $sformatf("sample=0x%0h latency=%0d", sample, latency);
  endfunction
endclass
