import "DPI-C" pure function longint clock();

class bvm_cover #( type T = int ) extends uvm_subscriber #(T);
  protected uvm_phase running_phase;
  protected real last_coverage = 0;
  longint last_clock = 0;

  typedef bvm_cover #(T) this_type;
  `uvm_component_param_utils(this_type)

  const static string type_name = 
    "bvm_cover #(T)";

  function new(string name, uvm_component parent);
    super.new(name, parent);
  endfunction

  virtual function string get_type_name();
    return type_name;
  endfunction

  task run_phase(uvm_phase phase);
    running_phase = phase;
    running_phase.raise_objection(this);
  endtask: run_phase

  virtual function void write(T t);
    real cov;
    longint clk;
    sample(t);
    clk = clock()/1000000;
    if(clk>last_clock) begin
      last_clock=clk;
      cov=get_coverage();
      if(cov>last_coverage) begin
        last_coverage = cov;
        `uvm_info(get_type_name(),$sformatf("Coverage: %2.0f%%",cov),UVM_LOW)
        if(cov>99 && running_phase!=null) begin
          running_phase.drop_objection(this);
          running_phase = null;
        end
      end
    end
  endfunction

  virtual function void sample(T t);
  endfunction

  virtual function real get_coverage();
    return 0.0;
  endfunction

  virtual function void report_phase(uvm_phase phase);
    super.report_phase(phase);
    `uvm_info(get_type_name(),
              $sformatf("Coverage: %2.0f%%",get_coverage()),
              UVM_LOW)
  endfunction

endclass

`define bvm_cover_utils(T, tr, cvrgrp) \
  virtual function void sample(T t); \
    tr = t; \
    cvrgrp.sample(); \
  endfunction \
  virtual function real get_coverage(); \
    return cvrgrp.get_inst_coverage(); \
  endfunction


