virtual class lpcm_sequence extends uvm_sequence #(lpcm_tr);

  function new(string name);
    super.new(name);
  endfunction

endclass

class lpcm_noise_seq extends lpcm_sequence;

  `uvm_object_utils(lpcm_noise_seq)

  function new(string name = "lpcm_noise_seq");
    super.new(name);
  endfunction
  
  virtual task body();
    forever begin
      `uvm_info(get_type_name(), "Generating transaction", UVM_HIGH)
      `uvm_do_with(req, {latency < 1000;});
      get_response(rsp);
    end
  endtask
endclass

import "DPI-C" pure function real sin(input real rTheta);

class lpcm_sine_seq extends lpcm_sequence;

  const real pi = 3.14159265;

  real sample_rate;

  rand int unsigned freq;
  rand int unsigned amplitude;
  rand int unsigned offset;
  rand int unsigned periods;
  rand int unsigned lat;

  `uvm_object_utils_begin(lpcm_sine_seq)
    `uvm_field_int(freq, UVM_DEFAULT | UVM_DEC)
    `uvm_field_int(amplitude, UVM_DEFAULT | UVM_DEC)
    `uvm_field_int(offset, UVM_DEFAULT | UVM_DEC)
    `uvm_field_int(periods, UVM_DEFAULT | UVM_DEC)
    `uvm_field_int(lat, UVM_DEFAULT | UVM_DEC)
  `uvm_object_utils_end

  constraint c {
    freq > 10; // log(10)*10
    freq < 43; // log(20000)*10
    amplitude <= 1000;
    offset <= 1000 - amplitude;
    periods > 1;
    periods < 10;
    lat > 1;
    lat < 20;
  }

  function new(string name = "lpcm_sine_seq");
    super.new(name);
    sample_rate = 100_000;
  endfunction
  
  function string convert2string;
     return $psprintf("freq=%5f ampl=%1.3f off=%1.3f periods=%2d lat=%3d",
                      10.0**(freq/10.0), amplitude/1000.0, offset/1000.0, periods, lat);
  endfunction: convert2string

  virtual task body();
    longint unsigned n = sample_rate/(10.0**(freq/10.0));
    real ampl = amplitude/1000.0 - 1.0/(1<<`LPCM_SIZE);
    real off = offset/1000.0 - 1.0/(1<<`LPCM_SIZE);
    for (longint unsigned i = 0; i < n*periods; i+=1) begin
      int lpcm_sample;
      lpcm_sample = (off + ampl*sin(2*pi*(i*1.0/n))) * (1<<(`LPCM_SIZE-1));
      `uvm_do_with(req, {latency < lat; lpcm_sample == sample;});
      get_response(rsp);
    end
  endtask

endclass

class lpcm_complex_sine_seq extends lpcm_sequence;
  `uvm_object_utils(lpcm_complex_sine_seq)

  real sample_rate;

  lpcm_sine_seq seq;

  function new(string name = "lpcm_complex_sine_seq");
    super.new(name);
    sample_rate = 100_000;
  endfunction

  function void new_seq();
    seq = lpcm_sine_seq::type_id::create("seq");
    seq.enable_recording("seq");
    seq.sample_rate = sample_rate;
  endfunction

  task body();
    int stim_file;
    int f,a;
    stim_file = $fopen("stim.txt", "r");
    if (!stim_file) `uvm_warning(get_type_name(),"Stimuli file open failed")
    else begin
      while(!$feof(stim_file) && $fscanf(stim_file, "%d %d", f, a)==2 ) begin
        new_seq();
        seq.freq = f;
        seq.amplitude = a;
        seq.offset = 0;
        seq.periods = 1;
        seq.lat = 2;
        seq.start(m_sequencer, this);
      end
    end
    forever begin
      new_seq();
      assert( seq.randomize() );
      seq.start(m_sequencer, this);
    end
  endtask

endclass

