//
//------------------------------------------------------------//
//   Copyright 2009-2012 Mentor Graphics Corporation          //
//   All Rights Reserved Worldwid                             //
//                                                            //
//   Licensed under the Apache License, Version 2.0 (the      //
//   "License"); you may not use this file except in          //
//   compliance with the License.  You may obtain a copy of   //
//   the License at                                           //
//                                                            //
//       http://www.apache.org/licenses/LICENSE-2.0           //
//                                                            //
//   Unless required by applicable law or agreed to in        //
//   writing, software distributed under the License is       //
//   distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR   //
//   CONDITIONS OF ANY KIND, either express or implied.  See  //
//   the License for the specific language governing          //
//   permissions and limitations under the License.           //
//------------------------------------------------------------//

#include "systemc.h"
#include "tlm.h"

struct lpcm_tr {
  int sample;
  unsigned int latency;
};

struct dpcm_tr {
  int sample;
  unsigned int latency;
};

#include "uvmc.h"

using namespace uvmc;
UVMC_UTILS_2 (lpcm_tr,sample,latency)
UVMC_UTILS_2 (dpcm_tr,sample,latency)

using namespace sc_core;
using namespace tlm;

#include "definitions.h"
#define CODED_MAX ((1<<(DPCM_SIZE-1))-1)
#define CODED_MIN (-(1<<(DPCM_SIZE-1)))

class refmod: public sc_module {

  public:
  sc_port<tlm_get_if<lpcm_tr> > in;
  sc_port<tlm_put_if<dpcm_tr> > out;

  SC_CTOR(refmod): in("in"),
                   out("out") {
    SC_THREAD(run);
  }

  void run() {
    lpcm_tr tr_in;
    dpcm_tr tr_out;
    int deco_data = 0;
    while(1) {
      tr_in = in->get();
      tr_out.sample = tr_in.sample - deco_data;
      if(tr_out.sample < CODED_MIN) tr_out.sample = CODED_MIN;
      if(tr_out.sample > CODED_MAX) tr_out.sample = CODED_MAX;
      deco_data += tr_out.sample;
      tr_out.latency = tr_in.latency;
      out->put(tr_out);
    }
  }
};

int sc_main(int argc, char* argv[]) 
{  
  refmod rfmd("rfmd");

  uvmc_connect(rfmd.in,"rfmd.in");
  uvmc_connect(rfmd.out,"rfmd.out");

  cout << "SystemC version " << sc_release() << sc_version() << endl;

  sc_start();
  return 0;
}

