`include "uvm_macros.svh"

`include "bvm_comparator.sv"

import uvmc_pkg::*;

class tb_env extends uvm_env;

  lpcm_agent mst;
  dpcm_agent slv;

  uvm_tlm_analysis_fifo #(lpcm_tr) fifo;
  bvm_comparator #(dpcm_tr) sb;
  
  `uvm_component_utils(tb_env)

  function new(string name, uvm_component parent = null);
    super.new(name, parent);
    fifo = new("fifo", this);  //  cannot use factory
  endfunction

  virtual function void build_phase(uvm_phase phase);
    super.build_phase(phase);
    mst = lpcm_agent::type_id::create("master", this);
    slv = dpcm_agent::type_id::create("slave", this);

    // get/set_config_* uses the underlying uvm_config_db structure which is
    // generally recommended for string and objects, but for numeric values
    // specifically it is an advantage to use get/set_config_int due possible
    // programming errors when using to the strong typing semantics of
    // uvm_config_db. Get more information here:
    // http://www.uvmworld.org/forums/archive/index.php/t-355.html
    mst.set_config_int("", "is_active", UVM_ACTIVE);
    slv.set_config_int("", "is_active", UVM_PASSIVE);
    slv.set_config_int("mon", "pipeline_latency", 1);
    
    sb = bvm_comparator#(dpcm_tr)::type_id::create("sb", this);
  endfunction

  virtual function void connect_phase(uvm_phase phase);
    super.connect_phase(phase);
    // Connect fifo
    mst.item_collected_port.connect(fifo.analysis_export);
    // Connect refmod
    uvmc_tlm1 #(lpcm_tr)::connect(fifo.get_export,"rfmd.in");
    // Connect scoreboard
    uvmc_tlm1 #(dpcm_tr)::connect(sb.from_refmod,"rfmd.out");
    slv.item_collected_port.connect(sb.from_dut);
  endfunction

  virtual function void end_of_elaboration_phase(uvm_phase phase);
    super.end_of_elaboration_phase(phase);
  endfunction
  
  virtual function void report_phase(uvm_phase phase);
    super.report_phase(phase);
    // UVM_NONE means print even with no verbosity
    `uvm_info(get_type_name(), $sformatf("Reporting matched %0d", sb.m_matches), UVM_NONE)
    if (sb.m_mismatches) begin
      `uvm_error(get_type_name(), $sformatf("Saw %0d mismatched samples", sb.m_mismatches))
    end
    // The way that the in_order_comparator works, there is no direct way of
    // seeing whether we have any orphaned transactions inside the comparator.

    // Now I have to keep track of how many transactions I've intended to send
    // through the DUT from the sequencer and match that with the number of
    // matched transactions seen in the scoreboard.

    // I hate to say it, but the VMM Datastream Scoreboard actually worked
    // better in the most basic case. There I could specify no orphans, no
    // mismatches, some transactions (which are fine heuristics) and be done
    // with it.
  endfunction
endclass
