`include "tb_env.sv"

// I don't think that there is a naming convention of the base test except
// ending with "test", thus I will make "tb_base_test" my naming convention
class tb_base_test extends uvm_test;
  tb_env env;
  lpcm_sequence seq;
  lpcm_cover cov_mst;
  dpcm_cover cov_slv;
  
  `uvm_component_utils(tb_base_test)

  function new(string name, uvm_component parent = null);
    super.new(name, parent);
  endfunction

  virtual function void build_phase(uvm_phase phase);
    super.build_phase(phase);
    // Instantiate environment
    env = tb_env::type_id::create("tb_env", this);
  endfunction

  virtual function void connect_phase(uvm_phase phase);
    super.connect_phase(phase);
    env.mst.item_collected_port.connect(cov_mst.analysis_export);
    env.slv.item_collected_port.connect(cov_slv.analysis_export);
  endfunction
 
  task run_phase(uvm_phase phase);
    seq.start(env.mst.sqr);
  endtask: run_phase

endclass

class tb_noise_test extends tb_base_test;
  `uvm_component_utils(tb_noise_test)

  function new(string name = "tb_noise_test", uvm_component parent = null);
    super.new(name, parent);
  endfunction

  virtual function void build_phase(uvm_phase phase);
    super.build_phase(phase);
    seq = lpcm_noise_seq::type_id::create("seq", this);
    cov_mst = lpcm_cover::type_id::create("cov_mst", this);
    cov_slv = dpcm_cover::type_id::create("cov_slv", this);
  endfunction
endclass

class tb_sine_test extends tb_noise_test;
  `uvm_component_utils(tb_sine_test)

  function new(string name = "tb_sine_test", uvm_component parent = null);
    super.new(name, parent);
  endfunction

  virtual function void build_phase(uvm_phase phase);
    super.build_phase(phase);
    seq = lpcm_complex_sine_seq::type_id::create("seq", this);
  endfunction
endclass


