module dut #(parameter LPCM_SIZE=16, DPCM_SIZE=12)
  (input logic  clk,
   input logic  resetb,

   // Master Interface
   input logic en_in,
   input logic signed [LPCM_SIZE-1:0] data_in,

   // Slave Interface
   output logic en_out,
   output logic signed [DPCM_SIZE-1:0] data_out
   );

  logic signed [LPCM_SIZE-1:0] deco_data;
  logic signed [LPCM_SIZE:0] unsat_data;
  logic signed [DPCM_SIZE-1:0] sat_data;
  
  always_comb
    unsat_data <= data_in - deco_data;

  `define DPCM_MIN -(1<<(DPCM_SIZE-1))
  `define DPCM_MAX (1<<(DPCM_SIZE-1))-1

  always_comb
    if(unsat_data>`DPCM_MAX)
      sat_data <= (DPCM_SIZE)'(`DPCM_MAX);
    else if(unsat_data<`DPCM_MIN)
      sat_data <= (DPCM_SIZE)'(`DPCM_MIN);
    else
      sat_data <= unsat_data;

  `undef DPCM_MIN
  `undef DPCM_MAX

  always @(posedge clk)
    if (!resetb) begin
      en_out <= 0;
      data_out <= 0;
      deco_data <= 0;
    end else begin
      if(en_in) begin
        data_out <= sat_data;
        deco_data <= deco_data + sat_data;
      end
      en_out <= en_in;
    end

endmodule 
